/*

 timeout.c - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "timeout.h"
#if defined _WIN32 || defined __CYGWIN__
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#if defined _WIN32
#include <mmsystem.h>
#else
#include <signal.h>
#include <unistd.h>
#endif

volatile int timeout_expired;
volatile int signal_received = 0;

#if defined _WIN32 || defined __CYGWIN__
void mdelay(unsigned ms) {
    SleepEx(ms, TRUE);
}
#endif
#ifdef _WIN32
static MMRESULT timer;
static MMRESULT period = TIMERR_NOCANDO;
#define RESOLUTION 100

static void CALLBACK timeout_callback(UINT id, UINT msg, DWORD dw0, DWORD dw1, DWORD dw2) {
    (void)id;
    (void)msg;
    (void)dw0;
    (void)dw1;
    (void)dw2;
    timeout_expired = 1;
}

void timeout_init(void) {
    period = timeBeginPeriod(RESOLUTION);
}

void timeout_set(unsigned int i) {
    if (timer != 0) timeKillEvent(timer);
    timer = timeSetEvent(i * 1000, RESOLUTION, timeout_callback, 0, TIME_ONESHOT | TIME_CALLBACK_FUNCTION);
}

void timeout_cancel(void) {
    if (timer != 0) {
        timeKillEvent(timer);
        timer = 0;
    }
}

void timeout_deinit(void) {
    if (timer != 0) {
        timeKillEvent(timer);
        timer = 0;
    }
    if (period == TIMERR_NOERROR) timeEndPeriod(RESOLUTION);
}
#else
static void timeout(int x) {
    (void)x;
    timeout_expired = 1;
}

void timeout_init(void) {
    signal(SIGALRM, timeout);
}

void timeout_deinit(void) {
    timeout_cancel();
    signal(SIGALRM, SIG_DFL);
}
#endif

